// -----------------------------------------------------------------------------
// Sylk4OOo
// A SYLK converter to OpenOffice.org Calc (all versions)
//
// version 1.01 - 2006-12-17
//
// (C) 2006 Jean-Francois Nifenecker - jean-francois.nifenecker@laposte.net
//
// -----------------------------------------------------------------------------
// LICENSE
// This software is released under the terms of the GNU LGPL.
// For more information, see the attaxhed License.txt file
// or http://www.gnu.org/licenses/lgpl.html
// -----------------------------------------------------------------------------
//
program Sylk4OOo;
{$APPTYPE CONSOLE}
uses
  // Delphi
  Classes, SysUtils,
  // Application
  FileUtils,
  Options,
  DisplayMsg
  ;

var
  // memory SYLK file storage
  g_Sylk  : TStringList;

const
  k_SYLKid      = 'ID;';
  k_MagicValue  = 'ID;PSCALC3';

const
  ERR_NOT_FOUND   = 1;
  ERR_NOT_A_SYLK  = 2;
  ERR_CANT_UPDATE = 3;
  ERR_CANT_RENAME = 4;
  ERR_NO_LIST     = 5;

resourcestring
  // standard messages
  k_HaltedMsg     = '  Process halted';
  k_RenamedMsg    = '  renamed as %s';
  k_UpdatedMsg    = '  successfully updated';
  k_ProcessingMsg = 'Processing: [%d] %s';
  k_ReadMsg       = 'File list read from: %s (%d files)';
  k_BuiltMsg      = 'File list built (%d files)';
  // error messages
  k_NotFoundMsg   = '  ERROR! File not found';
  k_NoSYLKMsg     = '  ERROR! File is not a SYLK file';
  k_NotRenamedMsg = '  ERROR! Can''t rename %s into %s';
  k_NotUpdatedMsg = '  ERROR! Can''t update %s';
  k_NoFileListMsg = 'ERROR! SYLK file list %s not found';

procedure HaltOnError(const aErrMsg: string; aErrCode: Integer);
begin
DisplayLn(aErrMsg);
if g_Options[optHaltOnError].Enabled
then
  begin
  DisplayLn(k_HaltedMsg);
  Halt(aErrCode);
  end;
end;// HaltOnError

// Checks the file is a SYLK file, ie the 1st line starts with "ID;"
function Check4SYLK(const aList : TStrings): Boolean;
begin
Result  :=  Copy(aList.Strings[0], 1, 3) = k_SYLKid;
end;// Check4SYLK

// Updates the SYLK file contents
function UpdateSylk(const aFileName: string): Boolean;
begin
g_Sylk.Clear;
g_Sylk.LoadFromFile(aFileName);
Result  :=  Check4SYLK(g_Sylk);
if Result
then
  begin
  g_Sylk.Strings[0] :=  k_MagicValue;
  end;
end;// UpdateSylk

function Check4Existence(const aFileName : string): Boolean;
begin
Result  :=  FileExists(aFileName);

if not Result
then
  begin
  HaltOnError(k_NotFoundMsg, ERR_NOT_FOUND);
  end;
end;// Check4Existence

// Converts an individual SYLK file to OOo
procedure ConvertFile(const aFileName : string);

var
  l_Bak : string;

begin
if Check4Existence(aFileName)
then
  begin
  if g_Options[optBackUp].Enabled
  then
    // backup option activated
    begin
    l_Bak :=  ChangeFileExt(aFileName, '.' + g_Options[optBackupExt].Value);
    if RenameFile(aFileName, l_Bak)
    then
      begin
      DisplayLn(Format(k_RenamedMsg, [l_Bak]));
      if UpdateSylk(l_Bak)
      then
        begin
        g_Sylk.SaveToFile(aFileName);
        DisplayLn(k_UpdatedMsg);
        end
      else
        begin
        HaltOnError(k_NoSYLKmsg, ERR_NOT_A_SYLK);
        end;
      end
    else
      begin
      // can't rename file
      HaltOnError(Format(k_NotRenamedMsg, [aFileName, l_Bak]), ERR_CANT_RENAME);
      end;
    end
  else
    // no backup
    begin
    if UpdateSylk(aFileName)
    then
      begin
      if DeleteFile(aFileName)
      then
        begin
        g_Sylk.SaveToFile(aFileName);
        DisplayLn(k_UpdatedMsg);
        end
      else
        begin
        // can't delete file
        HaltOnError(Format(k_NotUpdatedMsg, [aFileName]), ERR_CANT_UPDATE);
        end;
      end
    else
      begin
      HaltOnError(k_NoSYLKmsg, ERR_NOT_A_SYLK);
      end;
    end;
  end;// if check4existence
end;// ConvertFile

// convert the SYLK files for OOo
procedure ConvertFileList(const aFileList: TStrings);

var
  i : Integer;

begin
g_Sylk  :=  TStringList.Create;
try
  for i := 0 to aFileList.Count - 1 do
    begin
    DisplayLn(Format(k_ProcessingMsg, [i + 1, aFileList.Strings[i]]));
    ConvertFile(aFileList.Strings[i]);
    end;

finally
  g_Sylk.Free;
end;//
end;// ConvertFileList

// gets the SYLK file list
procedure GetFileList(var aList : TStringList);
begin
aList.Clear;

if g_Options[optFileList].Enabled
then
  begin
  if FileExists(g_Options[optFileList].Value)
  then
    begin
    aList.Clear;
    aList.LoadFromFile(g_Options[optFileList].Value);
    end
  else
    begin
    DisplayLn(Format(k_NoFileListMsg, [g_Options[optFileList].Value]));
    Halt(ERR_NO_LIST);
    end;
  DisplayLn(Format(k_ReadMsg, [g_Options[optFileList].Value, aList.Count]));
  end;

if g_Options[optFileSpec].Enabled
then
  begin
  aList.Clear;
  BuildFileList(g_Options[optFileSpec].Value, faArchive, aList);
  DisplayLn(Format(k_BuiltMsg, [aList.Count]));
  end;

end;// GetFileList

// gets and processes the file list
procedure RunProcess;

var
  l_List  : TStringList;

begin
l_List  :=  TStringList.Create;
try
  GetFileList(l_List);
  ConvertFileList(l_List);
finally
  l_List.Free;
end;
end;// RunProcess


// Program ---------------------------------------------------------------------

begin
if ParseParms
then
  // params are ok, go on
  begin
  DisplayHeader;
  RunProcess;
  end
else
  // incorrect params: display help screen
  begin
  DisplayHelp;
  end;// if ParseParms

end.
